/**
 * 
 */
package com.sap.prd.mobile.ios.mios;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FilenameUtils;

/**
 * 
 *
 */
public class FrameworkStructureValidator
{

  private final File fmwkDir;
  private List<String> errorMsgs;

  public FrameworkStructureValidator(File fmwkDir)
  {
    this.fmwkDir = fmwkDir;
  }

  /**
   * 
   * @return a non <code>null</code> list containing the validation errors. The list is empty if the
   *         framework has a valid structure.
   */
  public List<String> validate()
  {
    errorMsgs = new ArrayList<String>();
    if (!fmwkDir.isDirectory()) {
      errorMsgs.add("'" + fmwkDir.getAbsolutePath() + "' is not a directory");
    }
    else {
      validateSubdirExistence("Versions", "Versions/A", "Versions/A/Headers", "Versions/A/Resources");
      String fmwkName = FilenameUtils.removeExtension(fmwkDir.getName());
      validateLib("Versions/A/" + fmwkName);
      validateLink("Headers", "Versions/A/Headers");
      validateLink("Resources", "Versions/A/Resources");
      validateLink(fmwkName, "Versions/A/" + fmwkName);
      validateLink("Versions/Current", "Versions/A");
    }
    return errorMsgs;
  }


  private void validateSubdirExistence(String... subDirNames)
  {
    for (String subDirName : subDirNames) {
      File dir = new File(fmwkDir, subDirName);
      if (!dir.isDirectory()) {
        errorMsgs.add("Missing the required subdirectory '" + subDirName + "'");
      }
    }
  }

  private void validateLib(String relativeLibPath)
  {
    File libFile = new File(fmwkDir, relativeLibPath);
    if (!libFile.exists()) {
      errorMsgs.add("Missing the required library file '" + relativeLibPath + "'");
    }

  }

  private void validateLink(String from, String to)
  {
    File fromLink = new File(fmwkDir, from);
    File toFile = new File(fmwkDir, to);
    try {
      if (!isSymbolicLink(fromLink)) {
        errorMsgs.add("Expected inside the framework a symbolic link from '" + from + "' to '" + to + "'");
      }
      else if (!fromLink.getCanonicalPath().equals(toFile.getCanonicalPath())) {
        errorMsgs.add("The link '" + from + "' does not point to '" + to + "'");
      }
    }
    catch (IOException ioe) {
      errorMsgs.add("Could not validate the symbolic link from '" + from + "' to '" + to + "': " + ioe.getMessage());
    }


  }

  private boolean isSymbolicLink(File file)
  {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    PrintStream printStream = new PrintStream(baos);
    try {
      int result = Forker.forkProcess(printStream, file.getParentFile(), "test", "-L", file.getName());
      return result == 0;
    }
    catch (IOException e) {
      errorMsgs.add("Could not fork process to check for symbolic link '" + file.getAbsolutePath() + "': " + e);
      return false;
    }

  }


}
